module Qiniu2Upyun
  class Qiniu
    attr_reader :config

    HOST = 'http://rsf.qbox.me'
    NAME = 'qiniu'

    def initialize(config)
      @config = config
    end

    def name
      NAME
    end

    def list(params={})
      uri = get_list_uri(params)

      Net::HTTP.start(uri.host) do |http|
        request = Net::HTTP::Post.new(uri)

        set_headers(request, uri)

        response = http.request(request)

        if response.is_a?(Net::HTTPOK)
          JSON.parse(response.body)
        else
          raise ResponseError.new(response)
        end
      end
    end

    def download(key)
      uri = get_download_uri(key)

      Net::HTTP.start(uri.host) do |http|
        request = Net::HTTP::Get.new(uri)

        http.request(request) do |response|
          data = get_data(key, response)

          response.read_body { |chunk| data.write(chunk) }

          data.io.rewind

          return(data.io)
        end
      end
    end

    private

    def set_headers(request, uri)
      access_token = get_access_token(uri)

      request['Authorization'] = access_token
    end


    def get_data(key, response)
      length = response['Content-Length'].to_i

      bar = Utils.get_bar(length, 'Download')

      if length > 1024 * 1024
        Data.new(Tempfile.new(key), bar)
      else
        Data.new(StringIO.new, bar)
      end
    end

    def get_list_uri(params)
      default_params = { 
        bucket: config.bucket,
        limit: 100,
        marker: nil,
        prefix: nil
      }

      query = URI.encode_www_form(default_params.merge(params))

      uri = URI(URI.encode("#{HOST}/list?#{query}"))
    end

    def get_download_uri(key)
      url = "#{config.domain}/#{key}?e=#{Time.now.utc.to_i + 3600}"

      uri = URI(URI.encode(url))

      download_token = get_download_token(uri)

      url = url.concat("&token=#{download_token}")

      uri = URI(URI.encode(url))
    end

    def get_download_token(uri)
      get_sign(uri.to_s)
    end

    def get_access_token(uri, body=nil)
      "QBox #{get_sign("#{uri.request_uri}\n#{body}")}"
    end

    def get_sign(signing_str)
      digest = OpenSSL::Digest.new('sha1') 

      sign = OpenSSL::HMAC.digest(digest, config.secret_key, signing_str)

      encoding_sign = Base64.urlsafe_encode64(sign)

      "#{config.access_key}:#{encoding_sign}"
    end
  end
end
